'use strict'; // Import

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import util from 'util';
import assert from 'assert';
import colors from 'ansicolors';
import diffUtil from 'diff';

/** Alias for setTimeout with paramaters reversed. */
export function wait(delay, fn) {
  return setTimeout(fn, delay);
}
/** Whether or not stdout and stderr are interactive. */

export function isTTY() {
  return process.stdout && process.stdout.isTTY === true && process.stderr && process.stderr.isTTY === true;
}
/**
 * Return a stringified version of the value with indentation and colors where applicable.
 * Colors will be applied if the environment supports it (--no-colors not present and TTY).
 */

export function inspect(value) {
  var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  // If the terminal supports colours, and the user hasn't specified, then default to a sensible default
  var colors = isTTY() && process.argv.indexOf('--no-colors') === -1;
  var depth = 50; // Inspect and return using our defaults

  return util.inspect(value, _objectSpread({
    colors: colors,
    depth: depth
  }, opts));
}
/** Return a highlighted string of a difference. */

export function inspectDiff(diff) {
  var result = '';
  diff.forEach(function (part) {
    var value = part.value;

    if (part.added) {
      value = colors.open.black + colors.bgGreen(value) + colors.open.green;
    } else if (part.removed) {
      value = colors.open.black + colors.bgBrightRed(value) + colors.open.green;
    }

    result += value;
  });
  return colors.green(result);
}
/** Return the difference between the new data and the old data. */

export function diff(newData, oldData) {
  if (_typeof(newData) === 'object' && _typeof(oldData) === 'object') {
    return diffUtil.diffJson(oldData, newData);
  } else {
    var a = inspect(oldData, {
      colors: false
    });
    var b = inspect(newData, {
      colors: false
    });
    return diffUtil.diffChars(a, b);
  }
}
/** Return a highlighted comparison between the new data and the old data. */

export function compare(newData, oldData) {
  return inspectDiff(diff(newData, oldData));
}
/** Log the inspected values of each of the arguments to stdout */

export function log() {
  for (var i = 0; i < arguments.length; ++i) {
    /* eslint no-console:0 */
    console.log(inspect(i < 0 || arguments.length <= i ? undefined : arguments[i]));
  }
}
/** Output a comparison of the failed result to stderr */

export function logComparison(actual, expected, error) {
  var lines = ['------------------------------------', 'Comparison Error:', colors.green(error.stack || error.message || error), ''];
  lines.push('Comparison Diff:', compare(actual, expected), '');
  lines.push('Comparison Actual:', inspect(actual), '', 'Comparison Expected:', inspect(expected), '------------------------------------'); // Work for node

  if (process.stderr) {
    process.stderr.write(lines.join('\n') + '\n');
  } // Work for browsers
  else {
      console.log(lines.join('\n'));
    }
}
/** Same as assert.equal in that it performs a strict equals check, but if a failure occurs it will output detailed information */

export function equal(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;

  try {
    assert.equal(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Same as assert.deepEQual in that it performs a deep equals check, but if a failure occurs it will output detailed information */

export function deepEqual(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'deep equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;

  try {
    assert.deepEqual(actual, expected, testName);
  } catch (checkError) {
    logComparison(actual, expected, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if the actual result contains the expected result .*/

export function contains(actual, expected) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'contains assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  if (testName == null) testName = "Expected [".concat(actual, "] to contain [").concat(expected, "]");

  try {
    assert.ok(actual.indexOf(expected) !== -1, testName);
  } catch (checkError) {
    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Checks to see if an error was as expected, if a failure occurs it will output detailed information */

export function errorEqual(actualError, expectedError) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'error equal assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  var expectedErrorMessage, actualErrorMessage;

  if (expectedError) {
    if (expectedError instanceof Error) {
      expectedErrorMessage = expectedError.message;
    } else {
      expectedErrorMessage = expectedError;
      expectedError = new Error(expectedErrorMessage);
    }
  }

  if (actualError) {
    if (actualError instanceof Error) {
      actualErrorMessage = actualError.message;
    } else {
      actualErrorMessage = actualError;
      actualError = new Error(actualErrorMessage);
    }
  }

  try {
    if (actualErrorMessage && expectedErrorMessage) {
      contains(actualErrorMessage, expectedErrorMessage, testName);
    } else {
      equal(actualError, expectedError || null, testName);
    }
  } catch (checkError) {
    logComparison(actualError && (actualError.stack || actualError.message || actualError), expectedErrorMessage, checkError);

    if (next) {
      next(checkError);
      return;
    } else {
      throw checkError;
    }
  }

  if (next) next();
}
/** Generate a callback that will return the specified value. */

export function returnViaCallback(value) {
  return function () {
    return value;
  };
}
/** Generate a callback that will receive a completion callback and call it with the specified result after the specified delay. */

/* eslint no-magic-numbers:0 */

export function completeViaCallback(value) {
  var delay = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 100;
  return function (complete) {
    wait(delay, function () {
      complete(null, value);
    });
  };
}
/** Generate a callback that return an error instance with the specified message/error. */

export function returnErrorViaCallback() {
  var error = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'an error occured';
  return function () {
    if (error instanceof Error) {
      return error;
    } else {
      return new Error(error);
    }
  };
}
/** Generate a callback that throw an error instance with the specified message/error. */

export function throwErrorViaCallback() {
  var error = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'an error occured';
  return function () {
    if (error instanceof Error) {
      throw error;
    } else {
      throw new Error(error);
    }
  };
}
/** Generate a callback that will check the arguments it received with the arguments specified, if a failure occurs it will output detailed information. */

export function expectViaCallback() {
  for (var _len = arguments.length, expected = new Array(_len), _key = 0; _key < _len; _key++) {
    expected[_key] = arguments[_key];
  }

  return function () {
    for (var _len2 = arguments.length, actual = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      actual[_key2] = arguments[_key2];
    }

    return deepEqual(actual, expected);
  };
}
/** Generate a callback that will check the error (if any) it receives for the expected error (if any), if a failure occurs it will output detailed information. */

export function expectErrorViaCallback(expected) {
  var testName = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'expect error via callback assertion';
  var next = arguments.length > 2 ? arguments[2] : undefined;
  return function (actual) {
    return errorEqual(actual, expected, testName, next);
  };
}
/** Expect the passed function to throw an error at some point. */

export function expectThrowViaFunction(expected, fn) {
  var testName = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'expect error via function assertion';
  var next = arguments.length > 3 ? arguments[3] : undefined;
  var actual = null;

  try {
    fn();
  } catch (error) {
    actual = error;
  }

  errorEqual(actual, expected, testName, next);
}
/** Deprecated. Use {@link expectErrorViaFunction} instead. */

export var expectErrorViaFunction = expectThrowViaFunction;
/** Deprecated. Use {@link expectErrorViaFunction} instead. */

export var expectFunctionToThrow = expectThrowViaFunction;